/**
 * @(#)MainProgram.java
 *
 * Licensed Materials - Property of IBM
 *
 * 5724-K80
 *
 * Copyright IBM Corp. 2007 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * This is sample code made available for use in accordance with terms set
 * forth in the license agreement document for the WebSphere Remote Server.
 */
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Properties;

import com.ibm.jsdt.support.SupportBase;
import com.ibm.jsdt.support.SupportLinuxHelper;
import com.ibm.wrs.solution.utils.ISPUtils;
import com.ibm.wrs.solution.utils.MQUtils;
import com.ibm.wrs.solution.utils.Software;
import com.ibm.wrs.solution.utils.SoftwareLinuxHarness;
import com.ibm.wrs.solution.utils.WRSProperties;
import com.ibm.wrs.solution.utils.WRSUtils;

/**
 * This class is used to install WebSphere MQ 6.0.0.0
 */
public class MainProgram extends SoftwareLinuxHarness {
        @SuppressWarnings("unused")
        private static final String copyright0 = "Licensed Materials - Property of IBM";
        @SuppressWarnings("unused")
        private static final String copyright1 = "5724-K80";
        @SuppressWarnings("unused")
        private static final String copyright2 = "Copyright IBM Corp. 2007 All Rights Reserved.";
        @SuppressWarnings("unused")
        private static final String copyright3 = "US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.";

        /** Main rpms in a WebSphere MQ installation */
        private String[] rpmList;
        /* This function return the List of rpm to be install */
        protected String [] getRPMList(){
                if (WRSUtils.getProperty(sb, "installexplorer").equals("true")){
                        rpmList=new String []{
                                        "MQSeriesRuntime-6.0.0-0.i386.rpm",
                                        "MQSeriesSDK-6.0.0-0.i386.rpm",
                                        "MQSeriesServer-6.0.0-0.i386.rpm",
                                        "MQSeriesClient-6.0.0-0.i386.rpm",
                                        "MQSeriesJava-6.0.0-0.i386.rpm",
                                        "MQSeriesIES30-6.0.0-0.i386.rpm",
                                        "MQSeriesConfig-6.0.0-0.i386.rpm"
                                        };
                }
                else{
                        rpmList=new String [] {
                                        "MQSeriesRuntime-6.0.0-0.i386.rpm",
                                        "MQSeriesSDK-6.0.0-0.i386.rpm",
                                        "MQSeriesServer-6.0.0-0.i386.rpm",
                                        "MQSeriesClient-6.0.0-0.i386.rpm",
                                        "MQSeriesJava-6.0.0-0.i386.rpm",
                                        };
                }
                return rpmList;
        }
        /** Language codes for message bundles included with MQ */
        private String[] langs = {"de", "es", "fr", "it", "ja", "ko", "pt", "Zh_CN", "Zh_TW"};

        public MainProgram() {
                software = new Software("WebSphere MQ");
        }

        //Can't use parent's install method because we need to install via RPM
        protected int install(String command) {
                int rc = FAILURE;

                if (preInstall(this) == SUCCESS
                        && installMQRPMs() == SUCCESS
                        && postInstall(this) == SUCCESS) {
                        rc = SUCCESS;
                }
                else {
                        rc = FAILURE;
                }

                return rc;
        }

        protected int preInstall(SupportBase sb) {
                return addMQGroupsAndUsers();
        }

        protected int postInstall(SupportBase sb) {
                // Use custom properties object for substitutions in metadata
                String prodId = this.getHelper().getProductInstallingId(this);
                WRSProperties p = WRSProperties.getInstance();

                // get the subset of properties for this project
        Properties subProps = p.getFilteredPropertySubset(this.getHelper().getIbmNsiProperties(this), prodId);

        // append additional properties
                subProps.setProperty("installLocation", "/opt/mqm");  //rpm install path is hardcoded
                subProps.setProperty("sifHome", WRSUtils.getSifHome(sb));
                synchMetadata(subProps);
                return SupportBase.SUCCESS;
        }

        /**
     *
     * @param userId  - User whose password will be changed.
     * @param newPassword  - New password to use.
     * @param encryptionType  - "des", "md5", or "blowfish"
     * @return
     */
        private int changeUserPassword (String userId, String newPassword, String encryptionType) {
                int rc = SUCCESS;
                try {
                        WRSUtils.log( this, "ALG02003I", new String [] {encryptionType});
                        // Create a temp file containing one line with this format:
                        // <user ID>:<encrypted password>
                        String workDir = getHelper().getUnpackedDir(this);
                        String tempFilename = workDir + "chpasswd.input";
                        BufferedWriter w1 = new BufferedWriter(new FileWriter(tempFilename));
                        w1.write(userId + ":" + newPassword);
                        w1.close();
                        
                        String cmdFilename = workDir + "chpasswdCmd.sh";
                        BufferedWriter w2 = new BufferedWriter(new FileWriter(cmdFilename));
                        w2.write("#!/bin/bash\n");
                        w2.write("chpasswd -e <" + tempFilename + "\n");
                        w2.write("rc=$?\n");
                        w2.write("exit $rc\n");
                        w2.close();
                        WRSUtils.invokeCommand(this, "chmod +x " + cmdFilename);
                        
                        // Change the password
                        rc = WRSUtils.invokeCommand(this, cmdFilename);

                        // delete the temp files
                        File f1 = new File(tempFilename);
                        File f2 = new File(cmdFilename);                        
                        f1.delete();
                        f2.delete();
                        
                } catch (IOException ioEx) {
                        WRSUtils.log(this, "ALG10025E", new String [] {ioEx.getMessage()});
                        rc = FAILURE;
                }
                return rc;
        } // end changeUserPassword()


        /**
         * Preps the environment for MQ by creating the MQ user and user groups
         * @return SUCCESS if successful, FAILURE otherwise
         */
        private int addMQGroupsAndUsers() {

                class MqUserSetupException extends Exception {
                        static final long serialVersionUID = 1L;
                        MqUserSetupException() { super(); }
                        MqUserSetupException(String s) { super(s); }
                }

                int rc = SUCCESS;
                // These flags will be used to undo those things we have done IF something goes wrong
                boolean mqmGroupCreated = false;
                boolean mqbrkrsGroupCreated = false;
                boolean mqmUserCreated = false;
                boolean varmqmDirectoryCreated = false;
                boolean pathScriptCreated = false;

                String mqEncPassword = "";           // encrypted pw for user mqm
                String pathScript = "/etc/profile.d/wrs_mq.sh";     // File to update PATH environment variable
                String varmqmDirectory = "/var/mqm";
                String userAddCommandPrefix = "useradd -c MQSeries_User "
                        + "-d /home/mqm -m -g mqm "
                        + "-G mqbrkrs "
                        + " -s /bin/bash -p ";

                try {
                        SupportLinuxHelper slh = (SupportLinuxHelper)this.getHelper();
            // create the /var/mqm directory if it isn't already there
                        File f = new File(varmqmDirectory);
                        if (!f.exists()) {
                                rc = WRSUtils.invokeCommand(this, "mkdir -p "+ varmqmDirectory);
                                if (rc == 0) {
                                        varmqmDirectoryCreated = true;
                                } else {
                                        throw new MqUserSetupException(ISPUtils.getWrsMessage("ALG02002E"));
                                }
                        } else if (!f.isDirectory()) {
                                throw new MqUserSetupException(ISPUtils.getWrsMessage("ALG02003E"));
                        }

                        WRSUtils.log(this, "ALG02004I", null);
                        // Create mqm group
                        this.setGroupName("mqm");
                        if (!slh.doesGroupExist(this)) {
                                if (slh.addGroup(this)) {
                                        mqmGroupCreated = true;
                                } else {
                                        throw new MqUserSetupException(ISPUtils.getWrsMessage("ALG02004E"));
                                }
                        } else {
                                WRSUtils.log( this, "ALG02005I", null);
                        }
                        // Create mqbrkrs group
                        this.setGroupName("mqbrkrs");
                        if (!slh.doesGroupExist(this)) {
                                if (slh.addGroup(this)) {
                                        mqbrkrsGroupCreated = true;
                                } else {
                                        throw new MqUserSetupException(ISPUtils.getWrsMessage("ALG02005E"));
                                }
                        } else {
                                WRSUtils.log( this, "ALG02006I", null);
                        }

                        // Add root user to mqm group
                        // Could also be done with the Linux command: "usermod -Gmqm root"
                        this.setUserName("root");
                        this.setGroupName("mqm");
                        slh.addUserToGroup(this);

                        // Create file to update PATH env variable.  This file will be executed for all users when they login.
                        // All .sh files in the /etc/profile.d directory are executed by /etc/profile
                        try {
                                WRSUtils.log( this, "ALG02007I", new String [] {pathScript});
                                BufferedWriter w = new BufferedWriter(new FileWriter(pathScript));
                                w.write("export PATH=$PATH:/opt/mqm/bin\n");
                                w.write("export LIBPATH=$LIBPATH:/opt/mqm/lib\n");
                                w.write("export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:/opt/mqm/java/lib\n");
                                w.close();
                                pathScriptCreated = true;
                                WRSUtils.invokeCommand(this, "chmod +x " + pathScript);
                        } catch (IOException ioEx) {
                                WRSUtils.log(this, "ALG10026E", new String [] {ioEx.getMessage()});
                                rc = FAILURE;
                        }

                        String mqmPw = WRSUtils.getProperty(this, "mqmPw");  // get the MQ password entered by the user

                        // We use the useradd command to create the mqm user.  The useradd command wants a DES
                        // encrypted password.  A simple Perl script which calls the Linux crypt(3) function is used.
                        WRSUtils.log(this, "ALG02008I", null);
                        String wrsBaseHome = WRSProperties.getInstance().getProperty("WRSBASE", "installLocation");

                        StringBuffer output = new StringBuffer();
                        StringBuffer error = new StringBuffer();
                        try {
                                // Using the ISPUtils version of executeCommand because we don't want
                                // the password to be logged, and we need the output of the command.
                                String command = wrsBaseHome + "/utility/" + "sifPwEncrypt.pl ";
                                WRSUtils.log(sb,"ALG00000I",new String[]{command + "*********"});
                                rc = ISPUtils.executeCommand(command + mqmPw, null, null, -1, output, error);
                        }
                        catch(Exception e) {
                                WRSUtils.log(sb,"ALG10005E", null);
                                WRSUtils.log(sb,"ALG10006E",new String []{ e.getClass().getName(), e.getMessage()});
                                e.printStackTrace();
                                rc = FAILURE;
                        }
                        finally {
                                WRSUtils.log(sb,"ALG00030I",new String [] {"" + rc});
                                //WRSUtils.log(sb,"ALG00011I", new String [] {SupportBase.NEW_LINE + output.toString() + SupportBase.NEW_LINE});
                                //WRSUtils.log(sb,"ALG00010I", new String [] {SupportBase.NEW_LINE + error.toString() + SupportBase.NEW_LINE});
                                if (SUCCESS != rc) {
                                        throw new MqUserSetupException(ISPUtils.getWrsMessage("ALG02000E"));
                                }
                                mqEncPassword = output.toString();
                        }

                        // Create user using the DES encrypted password
                        this.setUserName("mqm");
                        if (!slh.doesUserExist(this)) {
                        WRSUtils.log(this, "ALG02009I", null);
                        String userAddcommand = userAddCommandPrefix + mqEncPassword  + " mqm";
                        WRSUtils.log(this, "ALG00000I", new String [] {userAddCommandPrefix + "******** mqm"});
                                WRSUtils.invokeCommand(this, userAddcommand);
                                if (0 == rc) {
                                        mqmUserCreated = true;
                                } else {
                                        throw new MqUserSetupException(ISPUtils.getWrsMessage("ALG02006E", new String[]{""+rc}));
                                }
                        } else {
                                // User mqm already exists.  Set the pw to the one the user gave us.
                                rc = changeUserPassword("mqm", mqEncPassword, "des");
                                if (0 != rc) {
                                        throw new MqUserSetupException(ISPUtils.getWrsMessage("ALG02007E"));
                                }
                        }

                        WRSUtils.log(this, "ALG02010I", null);
                        WRSUtils.invokeCommand(this, getHelper().getUnpackedDir(this) + "mqlicense.sh -accept");

                } catch (MqUserSetupException mqE) {
                        WRSUtils.log(this, "ALG02001E",new String [] {mqE.getMessage()});
                        if (mqmUserCreated) {
                                WRSUtils.invokeCommand(this, "userdel -r mqm");
                        }
                        if (mqmGroupCreated) {
                                WRSUtils.invokeCommand(this, "groupdel mqm");
                        }
                        if (mqbrkrsGroupCreated) {
                                WRSUtils.invokeCommand(this, "groupdel mqbrkrs");
                        }
                        if (varmqmDirectoryCreated) {
                                WRSUtils.invokeCommand(this, "rm -rf " + varmqmDirectory);
                        }
                        if (pathScriptCreated) {
                                WRSUtils.invokeCommand(this, "rm " + pathScript);
                        }
                        if (0 == rc) {
                                rc = FAILURE;
                        }
                }
                return rc;
        } // end addMQGroupsAndUsers()

        /**
         * Installs all the MQ rpms
         * @return SUCCESS if all rpms install successfully, false otherwise
         */
        private int installMQRPMs() {
                int rc = SUCCESS;
                rpmList=getRPMList();
                for(int i = 0; i < rpmList.length && SUCCESS == rc; i++) {
                        rc = WRSUtils.installRPM(this, rpmList[i]);
                        rc |= MQUtils.updateUninstallScript(sb, rpmList[i]);
                }

                //Install language packs
                for (int i = 0; i < langs.length; i++) {
                        if (WRSUtils.getProperty(this, langs[i]).equalsIgnoreCase("true")){
                                String rpm = "MQSeriesMsg_" + langs[i] + "-6.0.0-0.i386.rpm";
                                rc = WRSUtils.installRPM(this, rpm);
                                rc |= MQUtils.updateUninstallScript(sb, rpm);
                        }
                }
                return rc;
        }

        /**
         * The entry point for the MqLinux main program.
         * @param args Array with one argument
         * parm#0 = Version.Release.Modification.Fixpack to be installed.
         */
        public static void main(String[] args) {
                MainProgram sb = new MainProgram();

                if (WRSUtils.verifyInputArguments(sb, args, 1)) {
                        sb.setVersion(args[0]);
                }

            if (sb.install(null) == SUCCESS) {
                        String installedVersion = WRSUtils.getInstalledMQVersion(sb);
                        System.exit(sb.verify(installedVersion));
                }
                else {
                        System.exit(FAILURE);
                }
        }
}
